﻿namespace Plaza2WebinarSample
{
	using System;
	using System.Windows;
	using System.Windows.Controls;
	using System.Windows.Data;
	using System.Collections.ObjectModel;
	using System.ComponentModel;
	using System.Net;

	using Ecng.Collections;
	using Ecng.Common;
	using Ecng.Configuration;
	using Ecng.Xaml;

	using StockSharp.BusinessEntities;
	using StockSharp.Logging;
	using StockSharp.Plaza;
	using StockSharp.Xaml;

	public partial class MainWindow
	{
		public readonly PlazaTrader Trader = new PlazaTrader();
		readonly LogManager _logManager = new LogManager();

		private readonly ICollectionView _tradesView;
		private readonly ICollectionView _orderLogView;
		private readonly ObservableCollection<Trade>  _trades = new ObservableCollection<Trade>();
		private readonly ObservableCollection<OrderLogItem> _orderLog = new ObservableCollection<OrderLogItem>();

		private bool _isConnected;
		private bool _initialized;

		public MainWindow()
		{
			InitializeComponent();

			//Добавлем графический компонент LogControl в качестве получателя логов
			_logManager.Listeners.Add(new GuiLogListener(LogControl));

			//Добавляем PlazaTrader как источник логов
			_logManager.Sources.Add(Trader);

			//SecurityEditor испольует Unity для получения источника, из которого будет получен список доступных инструментов,
			//поэтому регистрируем соответствующий объект в контейнере Unity.
			ConfigManager.RegisterService<ITrader>(Trader);
			//устанавливаем источник, из которого будет получен список доступных портфелей.
			Portfolios.Trader = Trader;

			//запоминаем ссылку на представление для коллекции сделок
			_tradesView = CollectionViewSource.GetDefaultView(_trades);
			//Устанавливаем обработчик, который будет вызываться для каждого элемента
			//который будет добавлен в коллекцию _trades. С помощью этого обработчика
			//будет выполняться фильтрация данных, если обработчик возвращает true,
			//то соответствующая запись будет отображена в таблице, если flase - нет.
			_tradesView.Filter = obj =>
			{
				var trade = (Trade)obj;
				var volume = TradesMinVolume.Value.HasValue ? TradesMinVolume.Value.To<decimal>() : 1;

				return trade.Security == Securities.SelectedSecurity && trade.Volume >= volume;
			};
			//Указываем источник данных для отображения в таблице сделок
			Trades.ItemsSource = _trades;

			//запоминаем ссылку на представление для коллекции заявок
			_orderLogView = CollectionViewSource.GetDefaultView(_orderLog);
			_orderLogView.Filter = obj =>
			{
				var item = (OrderLogItem)obj;
				var volume = QuotesMinVolume.Value.HasValue ? QuotesMinVolume.Value.To<decimal>() : 1;

				//Поле Order.Comment будем использовать для более читаемого отображения статуса в таблице заявок.

				//Если поле OrderLogItem.Trade не пустое, то соответствующая строка ордерлога означает сведение заявок,
				//иначе, если OrderLogItem.State == Done, то соответствующая строка ордерлога означает снятие ранее выставленной заявки,
				//и в последнем случае - строка соответствует регистрации новой заявки.
				item.Order.Comment = item.Trade == null ? (item.Order.State == OrderStates.Done ? "Снятие" : "Регистрация") : "Сведение";

				return item.Order.Security == Securities.SelectedSecurity && item.Order.Volume >= volume;
			};
			//Указываем источник данных для отображения в таблице заявок
			OrderLog.ItemsSource = _orderLog;
		}

		private void ConnectBtn_OnClick(object sender, RoutedEventArgs e)
		{
			try
			{
				if (!_isConnected)
				{
					//очистить коллекции, т.к. в них могут быть данные с прошлого подключения
					_trades.Clear();
					_orderLog.Clear();

					// инициализируем параметры шлюза
					Trader.Address = Address.Text.To<IPEndPoint>();
					Trader.AppName = AppName.Text;

					Trader.UseLocalProtocol = true;

					//если указаны параметры авторизации, то заполняем соответствующие поля
					if (IsAutorization.IsChecked == true)
					{
						Trader.Login = Login.Text;
						Trader.Password = Password.Password;
					}

					//Очищаем список таблиц, которые наобходимо получить
					Trader.Tables.Clear();
					//и заполняем этот список в соответствии с выбранными таблицами в главном окне приложения.
					Trader.SyncTables(Tables.SelectedTables);

					//если подключение производится первый раз после запуска приложения,
					//то необходимо подписаться на все обработчики событий, которые мы будем обрабатывать.
					if (!_initialized)
					{
						_initialized = true;

						//Для того, чтобы не скачивать уже полученные данные при переконнекте, необходимо добавить нужные таблицы в PlazaRevisionManager.Tables.
						//Наиболее объемные таблицы - это таблицы сделок и ордерлог, поэтому для этих таблиц будем получать только новые данные.
						//(под новыми подразумеваются данные, которые начинаются с номера ревизии, который был сохранен при последнем запуске приложения)
						var revisionManager = Trader.StreamManager.RevisionManager;
						revisionManager.Tables.Add(Trader.TableRegistry.AnonymousOrdersLog);
						revisionManager.Tables.Add(Trader.TableRegistry.TradeFuture);

						// подписываемся на событие успешного соединения
						Trader.Connected += () =>
						{
							this.GuiAsync(() => ChangeConnectStatus(true));
							_logManager.Application.AddInfoLog("Подключение установлено.");

							//после того, как было выполнено подключение - запускам экспорт данных (списка инструментов, сделок и т.д.)
							Trader.StartExport();
						};

						// подписываемся на событие разрыва соединения
						Trader.ConnectionError += error => this.GuiAsync(() =>
						{
							this.GuiAsync(() => ChangeConnectStatus(false));
							_logManager.Application.AddInfoLog("Ошибка соединения: {0}", error);
						});

						// подписываемся на событие успешного отключения
						Trader.Disconnected += () =>
						{
							this.GuiAsync(() => ChangeConnectStatus(false));
							_logManager.Application.AddInfoLog("Подключение разорвано.");
						};
						// подписываемся на событие обработки данных
						Trader.ProcessDataError += error => _logManager.Application.AddInfoLog("Ошибка обработки данных: {0}", error);

						//подписываемся на событие получения новых сделок
						Trader.NewTrades += trades => this.GuiAsync(() =>
						{
							//добавляем пришедшие сделки в коллекцию, которая привязана к таблице на главном окне
							_trades.AddRange(trades);

							//если в таблице присутствует полоса прокрутки,
							//то автоматически прокручиваем ее в самый конец, чтобы видеть последние данные.
							var scroll = Trades.FindVisualChild<ScrollViewer>();
							if (scroll != null)
								scroll.ScrollToEnd();
						});

						//подписываемся на событие получения новых данных по ордерлогу
						Trader.NewOrderLogItems += items => this.GuiAsync(() =>
						{
							//добавляем пришедшие строчки ордерлога в коллекцию, которая привязана к таблице на главном окне
							_orderLog.AddRange(items);

							var scroll = OrderLog.FindVisualChild<ScrollViewer>();
							if (scroll != null)
								scroll.ScrollToEnd();
						});
					}

					//выполняем подключение к роутеру.
					Trader.Connect();
				}
				else
				{
					//если подключение было установлено ранее, то отключаемся от роутера.
					Trader.Disconnect();
				}
			}
			catch (Exception ex)
			{
				MessageBox.Show(this, ex.Message, "Ошибка");
			}
		}

		/// <summary>
		/// Изменить доступность полей для ввода параметров подключения
		/// и названия эелементов в соответствии с текущим статусом подключения.
		/// </summary>
		/// <param name="isConnected">True - если установлено подключени, иначе - false.</param>
		private void ChangeConnectStatus(bool isConnected)
		{
			_isConnected = isConnected;

			//изменяем заголовок кнопки строку статуса
			ConnectBtn.Content = isConnected ? "Отключиться" : "Подключиться";
			ConnectionStatus.Content = isConnected ? "Подключен" : "Не подключен";

			//разрешаем или запрещаем редактирование параметров в зависимости от состояния подключения
			//если подключение установлено, то редактирование параметров - запрещено.
			IsAutorization.IsEnabled = Tables.IsEnabled = !isConnected;
			AppName.IsEnabled = Address.IsEnabled = !isConnected;
		}

		private void TradesMinVolume_OnValueChanged(object sender, RoutedPropertyChangedEventArgs<object> e)
		{
			//При изменении значения минимального объема для таблицы сделок, необходимо
			//обновить представление, связанное с таблицей, чтобы скрыть те строки,
			//которые не подходят под новое условие и показать недостающие.
			if (_tradesView != null)
				_tradesView.Refresh();
		}

		private void QuotesMinVolume_OnValueChanged(object sender, RoutedPropertyChangedEventArgs<object> e)
		{
			//При изменении значения минимального объема для таблицы заявок, необходимо
			//обновить представление, связанное с таблицей.
			if (_orderLogView != null)
				_orderLogView.Refresh();
		}

		/// <summary>
		/// Зарегистрировать заявку для соответствующего направления.
		/// Цена, объем, инструмент и портфель берутся из соответствующих полей главного окна.
		/// </summary>
		/// <param name="direction">Направление заявки.</param>
		private void SendOrder(OrderDirections direction)
		{
			//В начале необходимо проверить что инструмент и портфель действительно выбраны.

			if (Portfolios.SelectedPortfolio == null)
			{
				MessageBox.Show("Не выбран портфель!");
				return;
			}

			if (Securities.SelectedSecurity == null)
			{
				MessageBox.Show("Не выбран инструмент!");
				return;
			}

			var security = Securities.SelectedSecurity;
			decimal price = 0;

			//Если в главном окне выбрана галочка market, то заявки будем выставлять по рынку (рыночными будем считать заявки выставленные по планкам)
			//в противном случае берем введенное пользователем значение цены.
			if (ByMarket.IsChecked == true)
				price = direction == OrderDirections.Buy ? security.MaxPrice : security.MinPrice;
			else if (OrderPrice.Value.HasValue)
				price = OrderPrice.Value.Value.To<decimal>();

			if (price == 0)
			{
				MessageBox.Show("Не указана цена!");
				return;
			}

			//создаем заявку со всеми заданными параметрами
			var order = new Order
			{
				Security = security,
				Portfolio = Portfolios.SelectedPortfolio,
				Price = price,
				Volume = OrderVolume.Value.HasValue ? OrderVolume.Value.Value.To<decimal>() : 1,
				Direction = direction,
			};

			//отправляем заявку на биржу
			Trader.RegisterOrder(order);
		}

		private void BuyClick(object sender, RoutedEventArgs e)
		{
			SendOrder(OrderDirections.Buy);
		}

		private void SellClick(object sender, RoutedEventArgs e)
		{
			SendOrder(OrderDirections.Sell);
		}
	}
}
